@@ -1,7 +1,28 @@
+/*
+ * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
+ */
+
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
+#if defined (__FreeBSD__) || defined (__OpenBSD__) || defined (__NetBSD__)
+#include <sys/time.h>
+#include <sys/resource.h>
+#endif
+
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
 #include <errno.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <fcntl.h>
+
 #ifdef HAVE_SYS_WAIT_H
 # include <sys/wait.h>
-#endif /* HAVE_SYS_WAIT_H */
+#endif
+
 #include "ecore_private.h"
 #include "Ecore.h"
 
@@ -47,6 +68,7 @@ static int          _ecore_exe_make_sure_its_dead(void *data);
 static int          _ecore_exe_make_sure_its_really_dead(void *data);
 static Ecore_Exe_Event_Add *_ecore_exe_event_add_new(void);
 static void         _ecore_exe_event_add_free(void *data, void *ev);
+static void         _ecore_exe_dead_attach(Ecore_Exe *exe);
 
 EAPI int            ECORE_EXE_EVENT_ADD = 0;
 EAPI int            ECORE_EXE_EVENT_DEL = 0;
@@ -206,6 +228,40 @@ _ecore_exe_check_errno(int result, const char *file, int line)
  * Functions that deal with spawned processes.
  */
 
+static int run_pri = ECORE_EXE_PRIORITY_INHERIT;
+
+/**
+ * Sets the priority at which to launch processes
+ *
+ * This sets the priority of processes run by ecore_exe_run() and
+ * ecore_exe_pipe_run(). If set to ECORE_EXE_PRIORITY_INHERIT child processes
+ * inherit the priority of their parent. This is the default.
+ *
+ * @param   pri value -20 to 19 or ECORE_EXE_PRIORITY_INHERIT
+ * @ingroup Ecore_Exe_Basic_Group
+ */
+EAPI void
+ecore_exe_run_priority_set(int pri)
+{
+   run_pri = pri;
+}
+
+/**
+ * Gets the priority at which to launch processes
+ *
+ * This gets ths priority of launched processes. See
+ * ecore_exe_run_priority_set() for details. This just returns the value set
+ * by this call.
+ *
+ * @return the value set by ecore_exe_run_priority_set()
+ * @ingroup Ecore_Exe_Basic_Group
+ */
+EAPI int
+ecore_exe_run_priority_get(void)
+{
+   return run_pri;
+}
+
 /**
  * Spawns a child process.
  *
@@ -295,7 +351,7 @@ ecore_exe_pipe_run(const char *exe_cmd, Ecore_Exe_Flags flags, const void *data)
    if ((flags & ECORE_EXE_PIPE_AUTO) && (!(flags & ECORE_EXE_PIPE_ERROR))
        && (!(flags & ECORE_EXE_PIPE_READ)))
      /* We need something to auto pipe. */
-     flags |= ECORE_EXE_PIPE_READ | ECORE_EXE_PIPE_ERROR; 
+     flags |= ECORE_EXE_PIPE_READ | ECORE_EXE_PIPE_ERROR;
 
    exe->child_fd_error = -1;
    exe->child_fd_read = -1;
@@ -339,11 +395,11 @@ ecore_exe_pipe_run(const char *exe_cmd, Ecore_Exe_Flags flags, const void *data)
      {
 	pid_t               pid = 0;
 	volatile int        vfork_exec_errno = 0;
-	
+
 	/* FIXME: I should double check this.  After a quick look around, this is already done, but via a more modern method. */
 	/* signal(SIGPIPE, SIG_IGN);    We only want EPIPE on errors */
 	pid = fork();
-	
+
 	if (pid == -1)
 	  {
 	     fprintf(stderr, "Failed to fork process\n");
@@ -351,6 +407,13 @@ ecore_exe_pipe_run(const char *exe_cmd, Ecore_Exe_Flags flags, const void *data)
 	  }
 	else if (pid == 0)	/* child */
 	  {
+	     if (run_pri != ECORE_EXE_PRIORITY_INHERIT)
+	       {
+		  if ((run_pri >= -20) && (run_pri <= 19))
+		    {
+		       setpriority(PRIO_PROCESS, 0, run_pri);
+		    }
+	       }
 	     /* dup2 STDERR, STDIN, and STDOUT.  dup2() allegedly closes the
 	      * second pipe if it's open. On the other hand, there was the
 	      * Great FD Leak Scare of '06, so let's be paranoid. */
@@ -392,7 +455,7 @@ ecore_exe_pipe_run(const char *exe_cmd, Ecore_Exe_Flags flags, const void *data)
 	     if (flags & ECORE_EXE_PIPE_WRITE)
 	       E_NO_ERRNO(result, close(writePipe[0]), ok);
 	     E_NO_ERRNO(result, close(statusPipe[1]), ok);
-	     
+
 	     _exit(-1);
 	  }
 	else			/* parent */
@@ -400,13 +463,13 @@ ecore_exe_pipe_run(const char *exe_cmd, Ecore_Exe_Flags flags, const void *data)
 	     /* Close the unused pipes. */
 	     E_NO_ERRNO(result, close(statusPipe[1]), ok);
 
-	     /* FIXME: after having a good look at the current e fd 
+	     /* FIXME: after having a good look at the current e fd
 	      * handling, investigate fcntl(dataPipe[x], F_SETSIG, ...) */
 	     /* FIXME: above F_SETSIG etc. - this is async SIGIO based IO
 	      * which is also linux specific so we probably don't want to
 	      * do this as long as select() is working fine. the only time
 	      * we really want to think of SIGIO async IO is when it all
-	      * actually works basically everywhere and we can turn all 
+	      * actually works basically everywhere and we can turn all
 	      * IO into DMA async activities (i.e. you do a read() then
 	      * the read is complete not on return but when you get a
 	      * SIGIO - the read() just starts the transfer and it is
@@ -421,7 +484,7 @@ ecore_exe_pipe_run(const char *exe_cmd, Ecore_Exe_Flags flags, const void *data)
 	     for (;;)
 	       {
 		  char                buf;
-		  
+
 		  E_NO_ERRNO(result, read(statusPipe[0], &buf, 1), ok);
 		  if (result == 0)
 		    {
@@ -518,7 +581,7 @@ ecore_exe_pipe_run(const char *exe_cmd, Ecore_Exe_Flags flags, const void *data)
 			     ok = 0;
 		       }
 		    }
-		  
+
 		  exes = _ecore_list2_append(exes, exe);
 		  n = 0;
 	       }
@@ -537,7 +600,7 @@ ecore_exe_pipe_run(const char *exe_cmd, Ecore_Exe_Flags flags, const void *data)
    else
      {
 	Ecore_Exe_Event_Add *e;
-	
+
 	e = _ecore_exe_event_add_new();
 	e->exe = exe;
 	if (e) /* Send the event. */
@@ -545,7 +608,7 @@ ecore_exe_pipe_run(const char *exe_cmd, Ecore_Exe_Flags flags, const void *data)
 			  _ecore_exe_event_add_free, NULL);
 	/* printf("Running as %d for %s.\n", exe->pid, exe->cmd); */
      }
-   
+
    errno = n;
    return exe;
 }
@@ -1006,6 +1069,40 @@ ecore_exe_continue(Ecore_Exe * exe)
    kill(exe->pid, SIGCONT);
 }
 
+/**
+ * Sends the given spawned process a interrupt (@c SIGINT) signal.
+ * @param   exe Process handle to the given process.
+ * @ingroup Ecore_Exe_Signal_Group
+ */
+EAPI void
+ecore_exe_interrupt(Ecore_Exe * exe)
+{
+   if (!ECORE_MAGIC_CHECK(exe, ECORE_MAGIC_EXE))
+     {
+	ECORE_MAGIC_FAIL(exe, ECORE_MAGIC_EXE, "ecore_exe_interrupt");
+	return;
+     }
+   _ecore_exe_dead_attach(exe);
+   kill(exe->pid, SIGINT);
+}
+
+/**
+ * Sends the given spawned process a quit (@c SIGQUIT) signal.
+ * @param   exe Process handle to the given process.
+ * @ingroup Ecore_Exe_Signal_Group
+ */
+EAPI void
+ecore_exe_quit(Ecore_Exe * exe)
+{
+   if (!ECORE_MAGIC_CHECK(exe, ECORE_MAGIC_EXE))
+     {
+	ECORE_MAGIC_FAIL(exe, ECORE_MAGIC_EXE, "ecore_exe_quit");
+	return;
+     }
+   _ecore_exe_dead_attach(exe);
+   kill(exe->pid, SIGQUIT);
+}
+
 /**
  * Sends the given spawned process a terminate (@c SIGTERM) signal.
  * @param   exe Process handle to the given process.
@@ -1014,25 +1111,12 @@ ecore_exe_continue(Ecore_Exe * exe)
 EAPI void
 ecore_exe_terminate(Ecore_Exe * exe)
 {
-   struct _ecore_exe_dead_exe *dead;
-
    if (!ECORE_MAGIC_CHECK(exe, ECORE_MAGIC_EXE))
      {
 	ECORE_MAGIC_FAIL(exe, ECORE_MAGIC_EXE, "ecore_exe_terminate");
 	return;
      }
-
-   dead = calloc(1, sizeof(struct _ecore_exe_dead_exe));
-   if (dead)
-     {
-	dead->pid = exe->pid;
-	dead->cmd = strdup(exe->cmd);
-	IF_FN_DEL(ecore_timer_del, exe->doomsday_clock);
-	exe->doomsday_clock =
-	   ecore_timer_add(10.0, _ecore_exe_make_sure_its_dead, dead);
-	exe->doomsday_clock_dead = dead;
-     }
-
+   _ecore_exe_dead_attach(exe);
    printf("Sending TERM signal to %s (%d).\n", exe->cmd, exe->pid);
    kill(exe->pid, SIGTERM);
 }
@@ -1385,7 +1469,7 @@ _ecore_exe_data_generic_handler(void *data, Ecore_Fd_Handler * fd_handler,
 	     lost_exe = 0;
 	     errno = 0;
 	     if ((num = read(child_fd, buf, READBUFSIZ)) < 1)
-	       /* FIXME: SPEED/SIZE TRADE OFF - add a smaller READBUFSIZE 
+	       /* FIXME: SPEED/SIZE TRADE OFF - add a smaller READBUFSIZE
 		* (currently 64k) to inbuf, use that instead of buf, and
 		* save ourselves a memcpy(). */
 	       {
@@ -1591,4 +1675,22 @@ _ecore_exe_event_del_free(void *data __UNUSED__, void *ev)
    free(e);
 }
 
+static void
+_ecore_exe_dead_attach(Ecore_Exe *exe)
+{
+   struct _ecore_exe_dead_exe *dead;
+
+   if (exe->doomsday_clock_dead) return;
+   dead = calloc(1, sizeof(struct _ecore_exe_dead_exe));
+   if (dead)
+     {
+	dead->pid = exe->pid;
+	dead->cmd = strdup(exe->cmd);
+	IF_FN_DEL(ecore_timer_del, exe->doomsday_clock);
+	exe->doomsday_clock =
+	   ecore_timer_add(10.0, _ecore_exe_make_sure_its_dead, dead);
+	exe->doomsday_clock_dead = dead;
+     }
+}
+
 #endif
